// rs4.cpp : Implementation of the Crs4 class
//
/*******************************************************************************
*                           OPEN SOURCE CODE DISCLAIMER                        *
* FILE NAME:          rs4.cpp                                                  *
* DATE:               December 2001                                            *
* PURPOSE:            Control a Leuze RotoScan 4 Laser Unit with a Serial Port *
* ORIGINAL CODE BY:   Juan Andrade Cetto & Albert Checa Puimedon               *
*                     (C) Institut de Robtica Industrial                      *
*                     Universidad Politcnica de Catalunya                     *
* ALGORITHM BASED ON: RS4_protocol_extern-e.doc (1.0) Leuze electronic         *
* CONTRIBUTOR(S):                                                              *
*                                                                              *
* The contents of this file are subject to the Mozilla Public License; you may *
* not use this file except in compliance with the License. You may obtain a    *
* copy of the License at http://www.mozilla.org/MPL/                           *
* Software distributed under the License is distributed on an "AS IS" basis,   *
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for *
* the specific language governing rights and limitations under the License.    *
*                                                                              *
* You may modify this code (or portions thereof) provided a copy of this       *
* disclaimer is included in the modified version.                              *
*******************************************************************************/
	

#include "serial.h"
#include "rs4.h"






/*JF: Funcion de sincronizacion con el flujo de datos*/
char sincronizar_jf(HANDLE portstream, Datos_RS4 *confi)
{

int  u  = 0;
int i=0, Thp=0, Pos_res=0, Tam_Inf=0;
unsigned char anByte;
char start_a,start_b,stop_a,stop_b,err;


/*JF: Se leen tres 0x00 seguidos que indican el fin de un paquete y dos 0x00 seguidos 
	   que indican el inicio del paquete, con eso se garantiza conocer el inicio.*/
printf("\n\n Sincronizando...!\n\n");


while(u<5)
	{    
			
	//err = serialbytesin(portstream, &anByte, 1, -1);
	lLastError = serial.Read(szBuffer,1,&dwBytesRead);
		
	if(err == FALSE) return(FALSE);
	
	if(anByte == 0x00)  u=u+1; 
	else u=0; 
	
	}  /*JF6: En este punto  se ha encontrado el inicio de un paquete*/
        
/*JF: Con el inicio se configuran los valores limites de la trama*/
u=0;
printf("\n\nLeyendo configuracion del RS4...!\n\n");

//JF:Primer paso: medir el tamao del paquete
 while(u<3)
	{   /*JF6 :Se lee un caracter a la vez.*/ 
		
		err = serialbytesin(portstream, &anByte, 1, -1);

		if(err == FALSE)  return(FALSE); 
		
		//De acuerdo al numero de campos de opcion se lee
		//la resolucion, segmento de start y de stop
		if(i==1) {
			Thp = anByte & 3;
			if(Thp>3) return(FALSE); //Error en la sincronizacion
			  else Pos_res=Thp+9;
			} 
        if(i== Pos_res)   confi->resolution=anByte;
		if(i== Pos_res+1) start_b=anByte;
		if(i== Pos_res+2) start_a=anByte;
		if(i== Pos_res+3) stop_b=anByte;
		if(i== Pos_res+4) stop_a=anByte;

        if(anByte == 0x00) u=u+1;
		else u=0; 

		i=i+1;	
	} 
//JF:Segundo paso: Calculo de limites del campo de info y Num Max de Char el valor de +2 o -2
//                 es debido a que no se cuentan los dos primeros 0x00, 0x00 del inicio del
//                 paquete .El valor 3-Thp, es en el caso de que el campo de opcion sea diferente de 3.

confi->Tam_info=(i+2-Tam_End-Tam_Hea_Info-(Tam_Hea_Paq -3 + Thp));
confi->Num_max_char=Tam_End + confi->Tam_info + Tam_Hea_Info + ((Tam_Hea_Paq -3 + Thp)-2);

//JF:Tercer Paso: Comparacion con los valores limites tomados de la trama:
confi->output_start=(int)(256*(int)start_b + (int)start_a);
confi->output_stop=(int)(256*(int)stop_b + (int)stop_a);
Tam_Inf=((confi->output_stop-confi->output_start)/confi->resolution)+1;

if(Tam_Inf*2 > confi->Tam_info) //Error en la sincronizacion
		{
		printf("Error de Sincronizacion, en los tamaos del campo de informacion !\n\n");
		printf("Tam_Info y Confi->Tam_info %d %d\n",Tam_Inf,(confi->Tam_info)/2);
		return(FALSE);
		} 

//JF: Cuarto Paso: Impresion de la configuracion del RS4
printf("\n\n***Configuracion del RS4****\n\n");
printf("Segmento Inicial (1 EQU -5,36 Grados): %d \n",confi->output_start);
printf("Segmento Final (529 EQU 185 Grados): %d\n",confi->output_stop);
printf("Mediciones tomadas: %d\n",confi->Tam_info);
printf("Resolucion (1 EQU 0,36 Grados) : %d\n\n\n",confi->resolution);

	
return(TRUE);    
}

/*JF: Funcion de lectura de la trama, permite leer la trama completa una vez el sistema se engancha*/

char leer_trama_jf(HANDLE portstream,Datos_RS4 *confi)
{
int k=0;
char err;
BOOL fin = FALSE;
double elapsed_time2;
clock_t start, finish;

start = clock();

		for (int i=0;i< confi->Num_max_char*2 && !fin;i++)	
		{ 
			//JF:Se lee el siguiente caracter
			err = serialbytesin(portstream, &confi->unbyte, 1, -1); 

			if(err == FALSE) { return(FALSE); }

            /*JF: se guarda cada caracter leido en la cadena mensaje*/   
			confi->mensaje[i]=confi->unbyte;
			
			/*JF: Con el fin de saber cuando se llega al 0x00,0x00,0x00 
			      o sea al final de un paquete */
			if( confi-> mensaje[i] == NULL)	//Se busca el fin de la cadena
				k=k+1;
			  else
				k=0;

			if(k>=3)
			{
				fin=TRUE;
				// En este momento se llega al final de un paquete
				// Este es el valor real con el cual termina el contador del for
				// y con el cual se realizan los calculos del contorno.
				confi->limit_k=i;

			} 
		}

finish = clock();
elapsed_time2 = (double)(finish-start)/CLOCKS_PER_SEC;

if(confi->limit_k + 1 > confi->Num_max_char + 2) { printf("Error Lectura de Trama, se leyeron mas de %d %s %d \n",confi->Num_max_char+2,": leidos",confi->limit_k+1); return(FALSE);}

printf("Bytes Leidos %d %s %6.4f %s",confi->limit_k+1," en :",elapsed_time2,"Seg.\n");

return(TRUE);
}


/*JF: Funcion para calcular las distancias a partir de los datos de usuario leidos de la trama*/

char proc_datos_jf(Datos_RS4 *confi)
{
	//UINT Suma=0;
	float Cte;
    
	//Se podria eliminar
	//resolution= confi->mensaje[(confi->limit_k)-4-(confi->Tam_info)-2*2]; 
	//output_start=(int)(256*(int)confi->mensaje[(confi->limit_k)-4-(confi->Tam_info)-2-1]+(int)confi->mensaje[(confi->limit_k)-4-(confi->Tam_info)-2] ); 
	//output_stop=(int)(256*(int)confi->mensaje[(confi->limit_k)-4-(confi->Tam_info)-1]+(int)confi->mensaje[(confi->limit_k)-4-(confi->Tam_info)]  );
	
	
	//version JF:
	Cte=ALLONES/TWOTOTHE15; //Expresion para convertir a metros la distancia. 
    for(int l=0;l<(confi->Tam_info/2);l++)
	{
    /*JF6: Los datos se guardan hacia atras,desde el 529 al 1.
	       Se empieza en cuatro por los tres tokens 0x00 de fin de cadena despues del
		   caracter de chequeo. Se inicia desde atras del paquete a recuperar los datos.
		   256 * mensaje(limit_k-4-(2*l+1)) corresponde al byte mas significativo 
		         mensaje(limit:k-4-2*1)/2  corresponde al byte menos significativo    
		   Division por dos de toda la expresion para eliminar el bit menos significativo del LSB    */
       confi->contourjf[l]=(float) ( Cte*(int)((256*(int)confi->mensaje[(confi->limit_k)-4 -(2*l+1)]+(int)confi->mensaje[(confi->limit_k)-4-2*l])/2 ) );
	}
	//printf("El valor limite del contador del maximo valor es: %d\n",*tam_info-1);
	
	printf("Medi No. 0 y No %d %s %f %f\n\n",confi->Tam_info/2-1,":",confi->contourjf[confi->Tam_info/2-1],confi->contourjf[0]);

return(TRUE);
}


