// rs4.h : Header file for rs4.cpp

#ifndef CRS4_H
#define CRS4_H


/*******************************************************************************
*                           OPEN SOURCE CODE DISCLAIMER                        *
* FILE NAME:          rs4.h                                                    *
* DATE:               December 2001                                            *
* VERSION:            0.02                                                     *
* PURPOSE:            Control a Leuze RotoScan 4 Laser Unit with a Serial Port *
* ORIGINAL CODE BY:   Juan Andrade Cetto & ALbert Checa Puimedon			   *   
*                     (C) Institut de Robtica Industrial                      *
*                     Universidad Politcnica de Catalunya                     *
* MODIFIED:			  Juan Fernando Florez M. Universidad del Cauca	           *
* DATE:				  Julio 2004											   *	
* ALGORITHM BASED ON: RS4_protocol_extern-e.doc (1.0) Leuze electronic         *
* CONTRIBUTOR(S):                                                              *
*                                                                              *
* The contents of this file are subject to the Mozilla Public License; you may *
* not use this file except in compliance with the License. You may obtain a    *
* copy of the License at http://www.mozilla.org/MPL/                           *
* Software distributed under the License is distributed on an "AS IS" basis,   *
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for *
* the specific language governing rights and limitations under the License.    *
*                                                                              *
* You may modify this code (or portions thereof) provided a copy of this       *
* disclaimer is included in the modified version.                              *
*******************************************************************************/



#include <time.h>
#include "serial.h"


//JF: Definicion de tamaos de encabezados y finales de trama maximos del RotoScan

#define MAX_INFO                  529 // Datos de medicion maximos de usuario que se pueden leer
#define MAX_TRAMA                 1081// Numero de caracteres  maximos en el paquete del RS4 to PC
#define Tam_End                  4 // 3 0x00,0x00,0x00 and check character
#define Tam_Hea_Info			 13// 2 bytes output stop,2 bytes output start, 1 resolution and 8 num_scan   	
#define Tam_Hea_Paq              6 // 3 bytes option (?), 1 byte command and 2 bytes start

#define PI						3.1415926535897

// Laser length parameters
#define ALLONES					65.0  // All ones corresponds to 65 meters
#define TWOTOTHE15				32768.0 // 2^15


/*JF: Definicin de una estructura de datos de configuracion del RS4, los primeros datos solo
  emplearan para configurar en forma automatica la lectura de los datos del paquete, a partir
  de la informacion de la resolucion, del segmento de arranque y de llegada.*/

struct Datos_RS4{ 

int resolution;
long output_start;
long output_stop;
long Num_max_char;
long Tam_info;


unsigned char unbyte;
long limit_k;


/*JF: En este arreglo se gurdan todos los caracteres deducidos de la trama del RS4*/
unsigned char mensaje[MAX_TRAMA]; // tamao del paquete del protocolo del RS4
/*JF: Arreglo donde se gurdan los datos del contorno */
float contourjf[MAX_INFO]; 

};

/*JF: Variables para almacenar datos claves tomados de la trama del RS4*/
// int output_start, output_stop, resolution;
	
	
	//Funciones RS4
	
	//! Permite realizar un enganche a la trama de datos
	//@param pstream
	//@param Datos_RS4 datos de configuracion de los datos enviados por el RS4
    //@param timeout
	//@returns TRUE if succesful
    char sincronizar_jf(portstream_fd, Datos_RS4*);

    //! Permite leer una trama una vez el sistema se engancha
	//@param pstream
    //@param Datos_RS4 datos de configuracion de los datos enviados por el RS4
	//@param buffer 
	//@param num_received_chars
    //@param timeout
	//@returns TRUE if succesful
    char leer_trama_jf(portstream_fd,Datos_RS4*);


	//! Permite procesar los datos de usuario sacados de la trama
	//@param Datos_RS4 datos de configuracion de los datos enviados por el RS4
    //@param num_received_chars
	//@returns TRUE if succesful
	char proc_datos_jf(Datos_RS4*);

	
 
#endif